#include <cyg/hal/hal_arch.h>
#include <cyg/kernel/kapi.h>
#include <cyg/infra/diag.h>

#include <stdio.h>
#include <iso646.h>
#include <stdint.h>
#include <stdlib.h>

#include "ezs_counter.h"
#include "ezs_gpio.h"
#include "ezs_stopwatch.h"

#define STACKSIZE CYGNUM_HAL_STACK_SIZE_MINIMUM+102400
static cyg_uint8 my_stack[STACKSIZE];
static cyg_handle_t thread_handle;
static cyg_thread threaddata;

#define DATA_SIZE 128
static uint32_t g_data[DATA_SIZE];

void print_measurement(cyg_uint32 timestamp, cyg_uint32 value) {
	const size_t buffer_length =
		10 /* first number */
		+ 1 /* separator */
		+ 10 /* second number */
		+ 1 /* newline  */
		+ 1 /* terminating null byte  */;
	char buffer[buffer_length];
	memset(buffer, 0, buffer_length);
	snprintf(buffer, buffer_length, "%u;%u\n", (unsigned int) timestamp, (unsigned int) value);
	diag_printf(buffer);
}

void swap(uint32_t data[], size_t first, size_t second)
{
	uint32_t tmp = data[first];
	data[first] = data[second];
	data[second] = tmp;
}

void quicksort(uint32_t a[], int n) {
    if (n < 2) {
        return;
		}
    int p = a[n / 2];
    uint32_t *l = a;
    uint32_t *r = a + n - 1;
    while (l <= r) {
        if (*l < p) {
            l++;
        } else if (*r > p) {
            r--;
        } else {
            uint32_t t = *l;
            *l = *r;
            *r = t;
            l++;
            r--;
        }
    }
    quicksort(a, r - a + 1);
    quicksort(l, a + n - l);
}


void heapsort(uint32_t arr[], unsigned int N)
{
	int t;			/* the temporary value */
	unsigned int n = N, parent = N / 2, index, child;	/* heap indexes */
	/* loop until array is sorted */
	while (1) {
/**
 * if part is executed N/2 times
 * else part is executed N times
 * => 1.5*N
 */
		if (parent > 0) {
			/* first stage - Sorting the heap */
			t = arr[--parent];	/* save old value to t */
		} else {
			/* second stage - Extracting elements in-place */
			--n;	/* make the heap smaller */
			if (n == 0) {
				return;	/* When the heap is empty, we are done */
			}
			t = arr[n];	/* save lost heap entry to temporary */
			arr[n] = arr[0];	/* save root entry beyond heap */
		}
		/* insert operation - pushing t down the heap to replace the parent */
		index = parent;	/* start at the parent index */
		child = index * 2 + 1;	/* get its left child index */
		while (child < n) {
			/* choose the largest child */
/*child >1 && n <= N => max loop N-1 */
			if (child + 1 < n && arr[child + 1] > arr[child]) {
				++child;	/* right child exists and is bigger */
			}
			/* is the largest child larger than the entry? */
			if (arr[child] > t) {
				arr[index] = arr[child];	/* overwrite entry with child */
				index = child;	/* move index to the child */
				child = index * 2 + 1;	/* get the left child and go around again */
			} else {
				break;	/* t's place is found */
			}
		}
		/* store the temporary value at its new location */
		arr[index] = t;
	}
}

void heapsort_job(void)
{
	heapsort(g_data, DATA_SIZE);
}

void quicksort_job(void)
{
	quicksort(g_data, DATA_SIZE - 1);
}

// A little test thread.
void thread(cyg_addrword_t arg)
{
	while (1) {
		cyg_thread_delay(10);	// Time base is set to 1 ms 

		// do things here...
#if 0
		heapsort_job();
#else
		quicksort_job();
#endif
	}
}

#define SAMPLE_SIZE 32
uint16_t g_sampled_values[SAMPLE_SIZE];

extern uint16_t sample_adc(void);
extern void initialize_adc(void);

void sample_job(void) {
	static bool initialized = false;
	static int position = 0;

	if (not initialized) {
		initialize_adc();
		initialized = true;
	}
	g_sampled_values[position] = sample_adc();
	++position;
}

void cyg_user_start(void)
{
	// Setup counter
	ezs_counter_init();
	ezs_gpio_init();

	size_t i;
	for (i = 0; i < DATA_SIZE; ++i) {
		g_data[i] = i;
	}

	// Create test thread
	cyg_thread_create(11, &thread, 0, "thread1", my_stack, STACKSIZE,
			&thread_handle, &threaddata);

	// and set thread ready to run
	cyg_thread_resume(thread_handle);
}
